import random # All luck, no skill.

# This program simulates the gameplay of a QTE-driven game.
# It supposes that there is no quicksave button (as usual) or even checkpoint between each QTE.
# If you pass a QTE, you can watch the next (unskippable) part of the cutscene, until the next QTE.
# If at any point you fail a QTE, your character is murdered horribly, you are treated to a game over screen and you have to restart the whole cutscene.
# If you reach the end of the cutscene, you're done. Also, the game is saved, and the cutscene therefore becomes skippable for you. (Except at this point you don't care anymore.)

plc = 1000 # Number of available zerglings
csc = 8    # Number of QTE-separated parts without a checkpoint (not made up - directly taken from the Resident Evil 4 knife fight cutscene)

# Sends a single (playtester) zergling with a skill level of p and reports how many times each sequence has been watched until he managed to reach the end.
def try_and_die(p):
	watchc = [0]*(csc+1)      # Take a notepad to keep track of the watch count.
	step = 1                  # Start at sequence N°1.
	while True:               # Until we're done...
		watchc[step] += 1       # > Watch the current sequence.

		if step == csc:         # > After the last sequence...
			break;                # >> ...mercifully, there is no QTE.

		if random.random() > p: # > QUICK QUICK QUICK PRESS X OR DIE ! Aw, snap, too late.
			watchc[0] += 1        # >> Watch the Game Over screen.
			step = 1              # >> Restart the whole cutscene.
		else:                   # > QUICK QUICK QUICK PRESS X OR DIE ! ...hey, that wasn't so hard. You made it !
			step += 1             # >> Now, let's get to the next sequence.
	return watchc             # Thank you for participating in this test. Please hand back your notepad.

# Sends a wave of zerglings at the cutscene (all of them, to get statistically significant results)
# Each zergling tries to pass the cutscene and reports how many times he has watched each sequence.
def zergling_rush(p):
	total_wc = [0]*(csc+1)           # Write down the results of the Swarm on a whiteboard.
	for pl in range(plc):            # For each zergling in the Swarm...
		new_watch = try_and_die(p)     # > Hand him a controller and get his notepad at the end.
		for c in range(csc+1):         # > For each row on his notepad...
			total_wc[c] += new_watch[c]  # >> Add the watch count to our total.
	return total_wc                  # Finally, rip the whiteboard off the wall and give it to your publisher.

# Takes the total watch count and displays how many times each sequence (including the Game Over screen) has been watched on average.
def verdict(watches):
	print ("A player that has a " + str(round(p*100)) + "% chance of passing a QTE has, on average, watched:")
	print ("Game Over \t- " + "{0:.2f}".format(watches[0]/plc) + " times.")
	for c in range(1,csc+1):
		print ("Sequence N°" + str(c) + "\t- " + "{0:.2f}".format(watches[c]/plc) + " times.")
	print ()

# We compute and display how many times on average each sequence has been watched by players who have an N% chance of succeeding at a QTE.
# N starts at 100% then decreases 5% by 5% until it reaches zero. (At which point - of course - we don't even try.)

# NOTE: You may have noticed that this code could very easily take advantage of multithreading; however it does not.
#       This is actually a feature. I want to still be able to use my computer while the zerglings are massively dying.

p = 1.00 # Probability of passing a single QTE (1.00 is Ray, 0.00 didn't even try)
while p > 0.00:             # While they technically still stand a chance...
	verdict(zergling_rush(p)) # > Run an experiment on our current batch of players and display the results.
	p -= 0.05                 # > Then get a batch of slightly less skilled players.

print("This statement never reached.")
# Bonus question: assuming that each sequence lasts 10" and the game over screen 5", how long on average before the 5%-chance zergling manages to complete the game ? (The rest of the game length being negligible.)

